/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menusub.h"
#include "lswinbase.h"
#include <lsapi.h>
#include <windows.h>
#include <string>
#include <functional>
#include <boost/shared_ptr.hpp>

class BangableMenu;
typedef boost::shared_ptr<BangableMenu> BangMenuPtr;

class BangableMenu
	:public Window
{
private:
	std::string m_bangcmd;
	bool m_autobang;

	void create(const wchar_t* title, HWND parent)
	{
		createWindow(WS_EX_TOOLWINDOW, title, WS_POPUP | WS_CLIPCHILDREN, 0, 0, 0, 0, parent);
	}
	
	void onEndSession(Message& message);
	void onSysCommand(Message& message);

	void onCommand(Message& message);
	void onTimer(Message& message);
	void onMenuRButtonUp(Message& message);
	void onEnterMenuLoop(Message& message);
	void onExitMenuLoop(Message& message);
	void onInitMenuPopup(Message& message);
	void onUnInitMenuPopup(Message& message);

protected:
	MenuSubPtr m_menu;

	BangableMenu(HWND parenthwnd);
	BangableMenu(const std::string& bang, const MenuSubPtr& sub, HWND parenthwnd, BangCommandEx* f);
	void windowProc(Message& message);
	
public:
	// Use shared_ptr to get copyable access to a noncopyable resource
	static BangMenuPtr createBangableMenu(const std::string& bang, const MenuSubPtr& sub, HWND parenthwnd, BangCommandEx* f)
	{
		BangMenuPtr temp(new BangableMenu(bang, sub, parenthwnd, f));
		return temp;
	}

	virtual ~BangableMenu();

	void onBang(const char* args);

	friend bool operator==(const BangMenuPtr& lhs, const std::string& rhs)
	{
		return lhs->m_bangcmd == rhs;
	}
};