/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "menuauto.h"
#include "globals.h"
#include <lsapi.h>


bool MenuAuto::autocolumns = false;

void MenuAuto::preDisplay(HWND hwnd)
{
	if (m_dirty)
	{
		parse();
		// this is the only place where m_dirty can every be set to false
		// only set to false for a non dynamic folder
		m_dirty = false;
	}
	// needs to be done everytime incase of a resolution change, or working area change
	autocolumn(hwnd);
}

void MenuAuto::postDisplay()
{
	if (m_dynamic || m_dirty)
	{
		int numleft = header ? 2 : 0;
		while(m_menu.size() > numleft)
		{
			try
			{
				m_menu.deleteItem(numleft, MF_BYPOSITION);
			}
			catch (std::runtime_error& e)
			{
				LSLog(LOG_DEBUG, MODULE_NAME, e.what());
			}
		}
		m_submap.clear();
		m_itemmap.clear();	
		m_dirty = true;
	}
}

void MenuAuto::autocolumn(HWND hwnd) 
// This code is based on the tallmenu SDK sample 
{
	if (!autocolumns)	// early out
		return;

	int itemcount = m_menu.size();
	
	POINT pt;
	GetCursorPos(&pt);
	RECT workarea;
	getWorkArea(&pt, &workarea);
	int screenheight = workarea.bottom - workarea.top - 10 * GetSystemMetrics(SM_CYEDGE);
	
	// use parent menu as it will have been shown before this menu, since GetMenuItemRect
	// only works on a menu that has been displayd
	int itemheight = 0;
	if (m_parent)
	{
		RECT itemsize = m_parent->getItemRect(hwnd, 0);
		itemheight = itemsize.bottom - itemsize.top;
	}
	// not an else if to protect against it haveing a parentmenu but that menu not being shown befor this one
	// not sure how that would happen but one 'if' == nothing
	if (itemheight == 0)
		itemheight = GetSystemMetrics(SM_CYMENU);
	
	for (int i = 0, currentheight = 0; i < itemcount; i++)
	{
		MENUITEMINFO mii;
		wchar_t buf[MAX_PATH];

		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_TYPE | MIIM_DATA;
		mii.dwTypeData = buf;
		mii.cch = MAX_PATH;
		try
		{
			m_menu.getItemInfo(i, MF_BYPOSITION, &mii);
		
			if (currentheight > screenheight)
			{
				currentheight = 0;
				mii.fType |= MFT_MENUBARBREAK;
			}
			else
			{
				mii.fType &= ~MFT_MENUBARBREAK;
			}

			m_menu.setItemInfo(i, MF_BYPOSITION, &mii);
		}
		catch (std::runtime_error& e)
		{
			LSLog(LOG_DEBUG, MODULE_NAME, e.what());
		}
		currentheight += itemheight;
	}
}