/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menuauto.h"
#include "idlist.h"
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
#include <vector>
#include <string>

class MenuFolder;
typedef boost::shared_ptr<MenuFolder> MenuFolderPtr;
typedef boost::weak_ptr<MenuFolder> MenuFolderWeakPtr;

class MenuFolder :
	public MenuAuto
{
public:
	class PathObject
	{
	public:
		std::wstring m_path;
		std::wstring m_file;

		PathObject(const std::wstring& directory, const std::wstring& pattern):
			m_path(directory),
			m_file(pattern)
		{
			validate();
		}

		PathObject(const std::wstring& completepath):
			m_path(completepath),
			m_file()
		{
			validate();
		}

		~PathObject()
		{	}

		void validate();
	};

	typedef std::vector<PathObject> PathVector;

private:
	typedef std::vector<MenuEntryPtr> EntryVector;
	typedef std::vector<MenuFolderPtr> FolderVector;
	typedef std::vector<std::wstring> StrVector;
	
	PathVector m_pathvect;

	MenuFolder(const std::wstring& txt, const std::wstring& pathname, bool isdynamic)
		:MenuAuto(txt, isdynamic)		
	{
		create(pathname);
	}

	MenuFolder(const std::wstring &pathname, bool isdynamic);

	void create(const std::wstring& path);
	void parseFolder(PathObject& parsepath, EntryVector& itemlist, FolderVector& sublist);
	void parse();
	void validatePath(StrVector& strs);
	
	// newpath must be a single path without a pattern
	void appendPath(std::wstring& newpath)
	{
		m_pathvect.push_back(PathObject(newpath));
		m_dirty = true;
	}

	static MenuFolderPtr createMenuFolderSpecial(const std::wstring& txt, const std::wstring& pathname, bool isdynamic)
	{
		MenuFolderPtr temp(new MenuFolder(txt, pathname, isdynamic));
		return temp;
	}

public:
	static MenuSubPtr createMenuFolder(const std::wstring& txt, const std::wstring& pathname, bool isdynamic)
	{
		MenuSubPtr temp(new MenuFolder(txt, pathname, isdynamic));
		return temp;
	}

	static MenuSubPtr createMenuFolder(const std::wstring& pathname, bool isdynamic)
	{
		MenuSubPtr temp(new MenuFolder(pathname, isdynamic));
		return temp;
	}

	virtual ~MenuFolder()
	{	}

	PathVector::const_iterator pathBegin() const
	{
		return m_pathvect.begin();
	}

	PathVector::const_iterator pathEnd() const
	{
		return m_pathvect.end();
	}

	// need to override the MenuSub function because folders have context menus
	bool displayContextMenu(HWND hwnd)
	{
		return util::ContextMenu::popupContextMenu(m_pathvect.front().m_path, hwnd);
	}
};