/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "menuitem.h"
#include "globals.h"
#include <shlwapi.h>
#include <shellapi.h>
#include <stdexcept>
#include <boost/scoped_array.hpp>
#include <lsapi.h>

bool MenuItem::hasextensions = true;

MenuItem::MenuItem(const std::wstring& path, const std::wstring& arguments)
	:MenuEntry(std::wstring()),
	m_file(path),
	m_args(arguments)
{
	std::wstring::size_type length = path.length();
	boost::scoped_array<std::wstring::value_type> working(new std::wstring::value_type[length + 1]);
	path.copy(working.get(), length);
	working[length] = L'\0';
	
	// I should consider writing my own version of this and PathRemoveExtension
	// to reduce dependence on Shlwapi
	PathStripPath(working.get());

	if (!hasextensions)
		PathRemoveExtension(working.get());
	else // handle shortcut extensions
	{
		std::wstring::pointer ext = working.get() + length - 4;
		if (lstrcmpi(ext, L".lnk") == 0)
			PathRemoveExtension(working.get());
	}

	setEntry(std::wstring(working.get()));
}
	
void MenuItem::execute(HWND hwnd)
{
	boost::scoped_array<std::wstring::value_type> working;
	std::wstring::size_type endfolder = m_file.find_last_of(L'\\');
	if (endfolder != std::wstring::npos)
	{
		working.reset(new std::wstring::value_type[endfolder + 1]);
		m_file.copy(working.get(), endfolder);
		working[endfolder] = L'\0';
	}

	SHELLEXECUTEINFO seinfo;
	ZeroMemory(&seinfo, sizeof(seinfo));
	seinfo.cbSize = sizeof(seinfo);
	seinfo.fMask = SEE_MASK_DOENVSUBST;
	seinfo.hwnd = hwnd;
	seinfo.lpFile = m_file.c_str();
	seinfo.lpParameters = m_args.c_str();	// need to insure that there double quotes are handled properly
	seinfo.lpDirectory = working.get();
	seinfo.nShow = SW_SHOW;
	if (!ShellExecuteEx(&seinfo))
		throw std::runtime_error(MAKE_ERROR_MSG("ShellExecuteEx Failed. " + getWinErrorMsg()));
}