/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menuentry.h"
#include "idlist.h"
#include <string>
#include <windows.h>

class MenuItem :
	public MenuEntry
{
private:
	std::wstring m_file;
	std::wstring m_args;

protected:
	// Only use with correct path, file name and arguments. Does no checking.
	MenuItem(const std::wstring& txt, const std::wstring& filename, const std::wstring& arguments)
		:MenuEntry(txt),
		m_file(filename),
		m_args(arguments)
	{	}

	// Builds entry, file, and arguments from given commandline.
	MenuItem(const std::wstring& path, const std::wstring& arguments);
	
public:	
	static bool hasextensions;
	
	static MenuEntryPtr createMenuItem(const std::wstring& txt, const std::wstring& filename, const std::wstring& arguments)
	{
		MenuEntryPtr temp(new MenuItem(txt, filename, arguments));
		return temp;
	}

    static MenuEntryPtr createMenuItem(const std::wstring& path, const std::wstring& arguments)
	{
		MenuEntryPtr temp(new MenuItem(path, arguments));
		return temp;
	}

	virtual ~MenuItem()
	{	}

	// These Methods are never used
	//wstring getFile() const
	//{
	//	return m_file;
	//}

	//wstring getArgs() const
	//{
	//	return m_args;
	//}

	//void setFile(wstring &newfile)
	//{
	//	m_file = newfile;
	//}

	//void setArgs(wstring &newargs)
	//{
	//	m_args = newargs;
	//}
		
	// Executes Item 
    void execute(HWND hwnd);

	// Displays Context Menu for the item
	bool displayContextMenu(HWND hwnd)
	{
		return util::ContextMenu::popupContextMenu(m_file, hwnd);
	}
};