/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menuentry.h"
#include <string>
#include <windows.h>
#include <boost/noncopyable.hpp>

class MenuMessage :
	public MenuEntry,
	public boost::noncopyable
{
private:
	wchar_t* m_windowclass;		// used to determine which window to send a message to.
	wchar_t* m_windowtitle;		// used to detrermine which window to send a message to.
	UINT m_msg;					// message
	WPARAM m_wparam;			// first message parameter
	LPARAM m_lparam;			// second message parameter

	void create(const wchar_t* wndclass, const wchar_t* wndtitle)
	{
		if (wndclass != NULL)
		{
			m_windowclass = new wchar_t[lstrlen(wndclass) + 1];
			lstrcpy(m_windowclass, wndclass);
		}
		else
			m_windowtitle = NULL;

		if (wndtitle != NULL)
		{
			m_windowtitle = new wchar_t[lstrlen(wndtitle) + 1];
			lstrcpy(m_windowtitle, wndtitle);
		}
		else
			m_windowtitle = NULL;
	}

protected:
	MenuMessage(std::wstring& txt, const wchar_t* wndclass, const wchar_t* wndtitle, UINT message, WPARAM wparam, LPARAM lparam)
		:MenuEntry(txt),
		m_windowclass(NULL),
		m_windowtitle(NULL),
		m_msg(message),
		m_wparam(wparam),
		m_lparam(lparam)
	{
		create(wndclass, wndtitle);
	}
	
public:
	static MenuEntryPtr createMenuMessage(std::wstring& txt, const wchar_t* wndclass, const wchar_t* wndtitle, UINT message, WPARAM wparam, LPARAM lparam)
	{
		MenuEntryPtr temp(new MenuMessage(txt, wndclass, wndtitle, message, wparam, lparam));
		return temp;
	}

	virtual ~MenuMessage()
	{
		delete [] m_windowclass;
		delete [] m_windowtitle;
	}

	void execute(HWND hwnd)
	{
		HWND msghwnd = FindWindow(m_windowclass, m_windowtitle);
    
		if (msghwnd != NULL)
			PostMessage(msghwnd, m_msg, m_wparam, m_lparam);
	}

	bool displayContextMenu(HWND hwnd)
	{
		return false;
	}
};
