/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "MenuSub.h"
#include "menu.h"
#include <windows.h>
#include <boost/utility.hpp>

MenuSubPtr createRemoteAmp(const std::wstring& label = REMOTEAMP);

class MenuSettings :
	public boost::noncopyable
{
private:
	std::wstring m_hotlist;
	
	// is the next element to start a newColumn
	bool m_column;

	// loads any settings 
	void settings(const char* prefix);
	
	// Step RC parsing worker methods
	void parseStepRC(const char* prefix, BangVector& vector, BangCommandEx* f, HWND parent);
	void parseFolder(const char* prefix, FILE *pStepRC, MenuSubPtr pop);
	bool parseLine(const char* prefix, FILE *pStepRC, MenuSubPtr pop, char* line);
	MenuSubPtr makeBangItem(const char* item, const std::wstring& title) const;
	MenuSubPtr makeBangItem(const char* item) const;

	void setColumn()
	{
		m_column = true;
	}

	bool getColumn()
	{
		bool rval = m_column;
		m_column = false;
		return rval;
	}

public:
	MenuSettings()
		:m_column(false)
	{	}

	virtual ~MenuSettings()
	{	}
	
	void configure(BangVector& vector, BangCommandEx* f, HWND parent);
};
