/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menuauto.h"
#include "menuidlist.h"
#include "idlist.h"
#include <boost/shared_ptr.hpp>
#include <string>
#include <vector>

class MenuShell
	:public MenuAuto,
	public util::ItemIdList
{
private:
	typedef boost::shared_ptr<MenuShell> MenuShellPtr;
	typedef std::vector<MenuShellPtr> ShellVector;
	typedef std::vector<MenuIdListPtr> IdListVector;

	MenuShell(int csidl)
		:MenuAuto(std::wstring(), true),
		ItemIdList(util::createVirtualFolder(csidl))
	{
		setEntry(getDisplayName());
	}

	MenuShell(const std::wstring& txt, int csidl)
		:MenuAuto(txt, true),
		ItemIdList(util::createVirtualFolder(csidl))
	{	}

	MenuShell(const std::wstring& txt, util::ItemIdList& idlist)
		:MenuAuto(txt, true),
		util::ItemIdList(idlist)
	{	}

	static MenuShellPtr createMenuShellSpecial(const std::wstring& txt, util::ItemIdList& idlist)
	{
		MenuShellPtr temp(new MenuShell(txt, idlist));
		return temp;
	}

	void parse();
	void parseFolder(ShellVector& folderlist, IdListVector& itemlist);

public:
	static MenuSubPtr createMenuShell(const std::wstring& txt, int csidl)
	{
		MenuSubPtr temp(new MenuShell(txt, csidl));
		return temp;
	}

	static MenuSubPtr createMenuShell(int csidl)
	{
		MenuSubPtr temp(new MenuShell(csidl));
		return temp;
	}

	virtual ~MenuShell()
	{	}

	bool displayContextMenu(HWND hwnd)
	{
		return util::ContextMenu::popupContextMenu(*this, hwnd);
	}
};
