/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once 
#include "menuentry.h"
#include "smarthmenu.h"
#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/noncopyable.hpp>
#include <windows.h>

class MenuSub;
typedef boost::shared_ptr<MenuSub> MenuSubPtr;

class MenuSub :
	public MenuEntry,
	public boost::noncopyable
{
private:
	typedef std::map <unsigned int, MenuEntryPtr> ItemMap;
	typedef std::map <HMENU, MenuSubPtr> SubMap;

	static long animation;
	static long position;

protected:
	// subclasses need to be able to access these for auto columns
	util::SmartHmenu m_menu;
	util::SmartHmenu* m_parent;

	// postDisplay needs access to these
	ItemMap m_itemmap;
	SubMap m_submap;
	
	explicit MenuSub(const std::wstring& txt);

	// Cleans up after preDisplay, not called directly, call endDisplay instead
	virtual void postDisplay();

public:
	static bool header;
	
	static MenuSubPtr createMenuSub(const std::wstring& txt)
	{
		MenuSubPtr temp(new MenuSub(txt));
		return temp;
	}

	virtual ~MenuSub()
	{	}

	// adds to menu
	virtual void addItem(const MenuEntryPtr& item, bool column = false);
	virtual void addSubMenu(MenuSubPtr& sub, bool column = false);
	virtual void addSeparator(bool column = false);
	virtual void addDeadItem(const std::wstring& txt, bool column = false);
	
	// Search for a contained item
	MenuEntryPtr getItem(unsigned int searchid) const;
	MenuSubPtr getSub(HMENU searchhmenu) const;

	// MenuEntry function overridden to allow for the header of the menu to change.
	void setEntry(std::wstring& txt);

	// displays the menu, Window messages must be processed by given hwnd
	void displayMenu(int x, int y, HWND hwnd);
	// called after display Menu returns
	void endDisplay();

	// called in response to some window messages most notably WM_INITMENU WM_ENDMENU
	// the calling is handled as such because of the order in which WM_COMMAND arrives
	// and the fact that any other behaviour would force BangableMenu to cache the 
	// displayed menus which is a waste
	// called for every menu
	virtual void preDisplay(HWND hwnd)
	{	}

	void execute(HWND hwnd)
	{	}

	bool displayContextMenu(HWND hwnd)
	{
		return false;
	}

	static void setAnimation(const char* horzmode, const char* vertmode);
	static void setPosition(const char* horzmode, const char* vertmode);
};