/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "menuauto.h"
#include "globals.h"
#include <string>
#include <stdexcept>

class MenuTasks :
	public MenuAuto
{
private:
	static BOOL WINAPI enumWindowsProc(HWND hwnd, LPARAM lparam);
	
protected:
	MenuTasks(const std::wstring& txt)
		:MenuAuto(txt, true)
	{	}

	MenuTasks()
		:MenuAuto(TASKS, true)
	{	}

	void parse()
	{
		if (!EnumWindows(enumWindowsProc, reinterpret_cast<LPARAM>(this)))
			throw std::runtime_error(MAKE_ERROR_MSG("EnumWindows Failed. " + getWinErrorMsg()));
	}

public:
	static MenuSubPtr createMenuTasks(const std::wstring& txt)
	{
		MenuSubPtr temp(new MenuTasks(txt));
		return temp;
	}

	static MenuSubPtr createMenuTasks()
	{
		MenuSubPtr temp(new MenuTasks());
		return temp;
	}

	virtual ~MenuTasks()
	{	}

	bool displayContextMenu(HWND)
	{
		return false;
	}
};