/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "globals.h"
#include <cstring>
#include <sstream>
using namespace std;

void getWorkArea(POINT* pt, RECT* prcWork)
{
    HMONITOR hmon;
    MONITORINFO moni;
    hmon = MonitorFromPoint(*pt, MONITOR_DEFAULTTONEAREST);
    moni.cbSize = sizeof(moni);
    GetMonitorInfo(hmon, &moni);
    *prcWork = moni.rcWork;
}

std::string getWinErrorMsg()
{
	DWORD errorcode = GetLastError();
	char* errstr = NULL;
	FormatMessageA( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		errorcode,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		errstr, 
		0,
		NULL
		);
	std::ostringstream oss;
	oss << "Error #" << errorcode << ": " << static_cast<char*>(errstr);
	LocalFree(errstr);
	return oss.str();
}

std::string makeErrorMsg(std::string msg, const char* file, int line)
{
	std::ostringstream oss;
	oss << msg << " [" << file << ", " << line << "]";
	return oss.str();
}

boost::shared_array<wchar_t> toUnicode(const char* astr)
{
	size_t astrsize = strlen(astr);
	int wstrsize = MultiByteToWideChar(CP_ACP, 0, astr + 1, astrsize, NULL, 0);
	boost::shared_array<wchar_t>wstr(new wchar_t[wstrsize + 1]);
	if (!MultiByteToWideChar(CP_ACP, 0, astr, astrsize + 1, wstr.get(), wstrsize + 1))
		throw std::runtime_error(MAKE_ERROR_MSG(std::string("Error converting \"") + astr + "\" to unicode. " + getWinErrorMsg()));
	return wstr;
}