/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once
#include "comhelpers.h"
#include <shtypes.h>
#include <shlobj.h>
#include <string>
#include <stdexcept>

namespace util
{
	InterfacePtr<IShellFolder> getDesktop();

	class ItemIdList
		:public IMallocPtr<ITEMIDLIST>
	{
	private:
		size_t sizeOf(ITEMIDLIST* pidl) const;
		ITEMIDLIST* file() const;
		ITEMIDLIST* copy(ITEMIDLIST* pidl) const;
		
		//short compare(const ItemIdList& idlist) const
		//{
		//	return static_cast<short>(HRESULT_CODE(getDesktop()->CompareIDs(0, m_ptr, idlist.m_ptr)));
		//}

		ITEMIDLIST* next(ITEMIDLIST* pidl) const
		{
			return reinterpret_cast<ITEMIDLIST*>(reinterpret_cast<char*>(pidl) + pidl->mkid.cb);
		}

	public:
		explicit ItemIdList(ITEMIDLIST* pidl = 0)
			:IMallocPtr<ITEMIDLIST>(pidl)
		{	}

		ItemIdList & append(const ItemIdList &idlist);
		ItemIdList getContainingElement() const;
		void execute(HWND hwnd);
		std::wstring getDisplayName() const;

		ItemIdList clone() const
		{
			return ItemIdList(copy(m_ptr));
		}
		
		ItemIdList & operator+=(const ItemIdList &idlist)
		{
			return append(idlist);
		}

		ItemIdList concat(const ItemIdList &idlist) const
		{
			ItemIdList rval = clone();
			rval.append(idlist);
			return rval;
		}

		ItemIdList operator+(const ItemIdList& idlist) const
		{
			return concat(idlist);
		}

		ItemIdList getLastElement() const
		{
			return ItemIdList(copy(file()));
		}
	};

	class ContextMenu
	{
	private:
		static IContextMenu2* pcm2;
		static LRESULT CALLBACK wndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

	public:
		static bool popupContextMenu(const ItemIdList &idlist, HWND hwnd);
		static bool popupContextMenu(const std::wstring &filesysobj, HWND hwnd);
	};

	ItemIdList createVirtualFolder(int csidl);
}