/*
 * Copyright 2003, Nathan E. Moore
 *
 * This file is part of Menu.
 *
 * Menu is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Menu is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Menu; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "menu.h"
#include "MenuSettings.h"
#include "globals.h"
#include <windows.h>
#include <algorithm>
#include <lsapi.h>

static Menu *menu; // The module
static unsigned int msgs[] = {LM_POPUP, LM_HIDEPOPUP, LM_GETREVID, 0};

//=========================================================
// Initialization and cleanup
//=========================================================
int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{ 
	Window::init(dllInst);
	try
	{
		menu = new Menu(ParentWnd);
	}
	catch (std::exception& e)
	{
		LSLog(LOG_DEBUG, MODULE_NAME, e.what());
	}
	return 0;
}

void quitModule(HINSTANCE dllInst)
{
	delete menu;
}

//=========================================================
// Bang commands
//=========================================================
void onBangCmdFunction(HWND, const char *pszName, const char *pszArguments)
{
	try
	{
		menu->onBangCmd(pszName, pszArguments);
	}
	catch (std::exception& e)
	{
		LSLog(LOG_DEBUG, MODULE_NAME, e.what());
	}
}

//---------------------------------------------------------
// Module constructor
//---------------------------------------------------------
Menu::Menu(HWND parentWnd)
	:BangableMenu(parentWnd)
{
	MenuSettings().configure(m_banglist, onBangCmdFunction, hWnd);
	//std::sort(banglist.begin(), banglist.end());

	PostMessage(hParent, LM_REGISTERMESSAGE, (WPARAM)hWnd, (LPARAM)msgs);
}

//---------------------------------------------------------
// Module destructor
//---------------------------------------------------------
Menu::~Menu()
{
	PostMessage(hParent, LM_UNREGISTERMESSAGE, (WPARAM)hWnd, (LPARAM)msgs);
}


//=========================================================
// Registered messages
//=========================================================

void Menu::windowProc(Message& message)
{
	BEGIN_MESSAGEPROC
		MESSAGE(onLMPopup, LM_POPUP)
		MESSAGE(onLMHidePopup, LM_HIDEPOPUP)
		MESSAGE(onGetRevId, LM_GETREVID)
		CHAIN_MESSAGEPROC(BangableMenu::windowProc)
	END_MESSAGEPROC
}


//=========================================================
// Message handlers
//=========================================================
void Menu::onLMPopup(Message& message) 
{
	onBangCmd("!popup", NULL);
}

void Menu::onGetRevId(Message &message)
{
	char* buf = reinterpret_cast<char*>(message.lParam);
	strcpy(buf, MODULE_NAME);
	strcat(buf, " ");
	strcat(buf, MODULE_VERSION);
	message.lResult = strlen(buf);
} 

void Menu::onLMHidePopup(Message &message)
{
	EndMenu();
}

//=========================================================
// Bang command handling
//=========================================================
void Menu::onBangCmd(const char* name, const char* args)
{
	BangVector::iterator iter = std::find(m_banglist.begin(), m_banglist.end(), std::string(name));
	if (iter != m_banglist.end())
		(*iter)->onBang(args);
}